/*****************************************************************************\

    Program :		CLRMAIN.PRG

    Purpose :		Interactive report design

    Authors / (C) :	Weigel, Huggle / Dr. Huggle & Partner GmbH

    Version / Date :	3.0 / 11.1990

    Language / Linker :	Clipper 5.0 / RTLink

    Compile /  Link :	MAKE-File CLREPORT.RMK for Clipper-utility RMake
			LINK-File CLREPORT.LNK for Overlay-Linker RTLink

    Source files :	CLREPGEN.CH  - header file

			CLRMAIN.PRG  - main module
			CLRPRINT.PRG - printing of report
			CLRGLOB.PRG  - global functions
			CLRERROR.PRG - errorsys

			CLREDIT.PRG  - report editor
			CLRLINE.PRG  - line data editing
			CLRFIELD.PRG - field data editing
			CLRGROUP.PRG - group data editing
			CLRDBF.PRG   - dbf definition
			CLROUT.PRG   - printing of report definition

			CLRUTIL.PRG  - supplemental functions

    For detailed information refer also to the MAKE-file CLREPORT.RMK and
    the LINK-file CLREPORT.LNK

\*****************************************************************************/

#include "clrepgen.ch"
#include "cltext.ch"


/* declarations */

memvar V_LNAME, V_RNAME


/* constants */

static HelpFile := "clr_help"		/* name of help file */

static ColorMono := {			/* color set monochrom */ ;
	"N/W,N/W,,W,N/W,", ;		// frame
	"N/W,+W/N,,,W/N,+W/N,+W/N", ;	// help
	"*N/W,+W/N,,,*+N/W,", ;		// main menu
	"W/N,+W/N,,,U/N,+W/N,+W/N,N/W,N/W,N/W,W/N", ;
	;				// mini-editor
	"W/N,+W/N,,,U/N,+W/N,+W/N,N/W,N/W,N/W,W/N", ;
	;				// report runtime
        "W/N,N/W,,,W/N,", ;		// list
	"W/N,W/N,,,+W/N,+W/N,U/N,N/W,N/W,,,,,,,,,,*+W/N", ;
	;				// report editor
	"+W/N,*N/W,,,W/N,", ;		// line type editing
	"*N/W,W/N,,,*N/W,", ;		// query
	"N/W,+W/N,,,N/W,", ;		// selection
	"N/W,+W/N,,,N/W,", ;		// mask
	"N/W,+W/N,,,N/W,", ;		// accept
	"N/W,", ;			// messages
	"+W/N,N/W,,,W/N,", ;		// error messages  
	"+W/N,N/W,,,W/N,", ;		// warning messages
	"N/W,+W/N,,,N/W," ;		// extra
	}


static ColorColor := {			/* color set color */ ;
	"+GB/B,N/W,,W,B/W,", ;		// frame
	"N/G,+W/N,,,+GB/G,+W/G,+G/G", ;	// help
	"*N/W,+GR/N,,,*W/W,", ;		// main menu
	"+GB/B,+W/B,,,N/GB,+GR/B,+G/B,+R/B,+RB/B,N/RB,*B/GB,*+GB/B,*+GB/B,", ;
	;				// mini-editor
	"N/W,+W/W,,,N/GB,+GR/W,+GB/W,R/W,RB/W,*W/N,*N/W,*+B/N,*+B/N,*+B/N,", ;
	;				// report runtime
        "+GB/B,+GR/GB,,,GB/B,", ;	// list
	"G/N,G/N,,,+GB/N,W/N,+GR/N,R/N,RB/N,,,,,,,,,,*+G/N,", ;
	;				// report editor
	"+GR/N,*N/W,,,W/N,", ;		// line type editing
	"+W/R,N/W,,,+W/R,", ;		// query
	"N/GB,+GR/N,,,N/GB,", ;		// selection
	"N/W,+GR/N,,,+W/W,", ;		// mask
	"N/W,+GR/N,,,+W/W,", ;		// accept
	"*+W/N,", ;			// messages
	"*+W/R,N/W,,,+W/R,", ;		// error messages  
	"+W/R,N/W,,,+W/R,", ;		// warning messages
	"+W/RB,+GR/GB,,,W/RB," ;	// extra
	}


function CLReport (cCmdLine)

#ifdef GERMAN
  static aMainMenu := { "<Bibliothek>", ;
			"Whlen", ;
			"Anlegen", ;
			"Umbenennen", ;
			"Lschen", ;
			"Reorganisieren", ;
			"<Report>", ;
			"Holen", ;
			"Neu", ;
			"Umbenennen", ;
			"Kopieren", ;
			"Lschen", ;
			"Ausgeben", ;
			"<Global>", ;
			"Seitenformat", ;
			"Drucker", ;
			"Zustze", ;
			"", ;
			"Ende"              }
#else
  static aMainMenu := { "<Libraries>", ;
			"Choose", ;
			"New", ;
			"Rename", ;
			"Erase", ;
			"Reorganize", ;
			"<Report>", ;
			"Select", ;
			"New", ;
			"Rename", ;
			"Copy", ;
			"Delete", ;
			"Output", ;
			"<Global>", ;
			"Page format", ;
			"Install Printer", ;
			"Supplement", ;
			"", ;
			"Quit"              }
#endif
  local cClEnv, cLibrary := [], aAva := {}, nChoice := 1, lMore := .t., ;
	svError, svEnv, svSelect, svKeyHelp
  local aMainCtrl := {		/* function, available, help */ ;
	{ {|| NIL },          {|| .f. },           NIL           }, ;
	{ {|| LibGet () },    {|| .t. },           "LIB GET"     }, ;
	{ {|| LibNew () },    {|| .t. },           "LIB NEW"     }, ;
	{ {|| LibRen () },    {|| .t. },           "LIB REN"     }, ;
	{ {|| LibDel () },    {|| .t. },           "LIB DEL"     }, ;
	{ {|| LibPack () },   {|| V_LNAME != [] }, "LIB PACK"    }, ;
	{ {|| NIL },          {|| .f. },           NIL           }, ;
	{ {|| ReportGet () }, {|| V_LNAME != [] }, "REPORT GET"  }, ;
	{ {|| ReportNew () }, {|| V_LNAME != [] }, "REPORT NEW"  }, ;
	{ {|| ReportRen () }, {|| V_LNAME != [] }, "REPORT REN"  }, ;
	{ {|| ReportCopy() }, {|| V_LNAME != [] }, "REPORT COPY" }, ;
	{ {|| ReportDel () }, {|| V_LNAME != [] }, "REPORT DEL"  }, ;
	{ {|| ReportOut () }, {|| V_LNAME != [] }, "REPORT EXEC" }, ;
	{ {|| NIL },	      {|| .f. },           NIL           }, ;
	{ {|| GlobPage () },  {|| .t. },           "CALL PAGE"   }, ;
	{ {|| GlobPrint () }, {|| .t. },           "CALL PRINT"  }, ;
	{ {|| GlobGlob () },  {|| .t. },           "CALL GLOB"   }, ;
	{ {|| NIL },	      {|| .f. },           NIL           }, ;
	{ {|| lMore := .f. }, {|| .t. },	   NIL           } }

  public CLRCOLOR := NIL		// color set (Mono or Color)

  /* declaration of private variables */
  private V_RNAME := [], V_LNAME := []	// current lib- and report name

  private V_HSTW  := "MENU MAIN"	// current help token

  svError := ErrorBlock ({|oError| ClrErrorSys (oError, svError)})

  svEnv := Environ ()			/* set up and save environment */
  svSelect := Select () ; svKeyHelp := SetKey (K_F1)

  if ( ValType (cCmdLine) != 'C' ) ; cCmdLine := [] ; end
  CLRCOLOR := iif (IsColor () .and. ;
		   ! Upper (AllTrim (cCmdLine)) $ "/M/BW-M-BW", ;
		   ColorColor, ColorMono)

  if ( ! Empty (cClEnv := gete ("CLIPPER")) .and. "S1" $ cClEnv )
    AEval (CLRCOLOR, {|cColor,i| CLRCOLOR[i] := StrTran (cColor, "*")})
    NoSnow (.t.)
  end

  SetColor (CLRCOLOR[_C_FRAME]) ; Border () ; ShowState (V_LNAME, V_RNAME)

  SetKey (K_F1, { |cProc, cLine, cVar| Help (cProc, cLine, cVar) })
  AEval (aMainCtrl, { |aControl| Aadd (aAva, aControl[2]) })

  repeat
    @ 1, 0, MaxRow () - 1, MaxCol () box ""

    ClrWindow (2, 4, 22, 23, CLRCOLOR[_C_MENUMAIN])

    repeat
      if ( cLibrary != V_LNAME )
        nChoice := iif (V_LNAME == [], 2, iif (LIB->(LastRec ()) == 0, 9, 8))
        ShowState (V_LNAME, V_RNAME)
      end

      V_HSTW := "MENU MAIN"

      while ( (nChoice := ClrAchoice (/* win */, .f., CLRCOLOR[_C_MENUMAIN], ;
				   /* head */, /* foot */, { nChoice, NIL }, ;
				   aMainMenu, aAva)) == 0 )
        nChoice := 19 ; end

      V_HSTW := aMainCtrl[nChoice][3]

      Eval (aMainCtrl[nChoice][1])
    until ( ! Empty (V_RNAME) .or. ! lMore )

    ClrWindow ()	/* menu window */

    if ( lMore )
      ShowState (V_LNAME, V_RNAME)

      ClrEditor (V_LNAME, V_RNAME)

      LIB->(Seek (V_RNAME)) ; V_RNAME := []
      while ( ! LIB->(RLock ()) ) ; end
      LIB->actual2 := .f. ; LIB->(Unlock ())

      commit
    end

    ShowState (V_LNAME, V_RNAME)
  until ( ! lMore )

  CloseDbf ("LIB") ; SetKey (K_F1, svKeyHelp)
  Select (svSelect) ; Environ (svEnv)

  ErrorBlock (svError)

return NIL


**********< Choose Library >***************************************************

static func LibGet ()

#ifdef GERMAN
  static cHead  := "Eingabe: Pfad und Name der Bibliothek > F10 Liste", ;
	 cMess  := "Fehlender Index/falscher Schlssel; Reindizierung", ;
         cErr1  := "Keine Report - Bibliothek (bitte Handbuch lesen !)", ;
         cErr2  := "Keine Datenbank / Datenbank im Eimer / keine Rechte"
#else
  static cHead  := "Input: Path and Filename > F10 Selection", ;
	 cMess  := "Missing index / incorrect index key; reindexing", ;
         cErr1  := "No report library file (please read the manual)", ;
         cErr2  := "No database file or database corrupted / no rights"
#endif
  static aChkStru := { ;
	"REPORT",     "DBF_ALIAS",  "FIELD_NO",   "FIELD_NAME", ;
	"FIELD_EXPR", "FIELD_TYPE", "FIELD_LEN",  "FIELD_DEC",  ;
	"FORM_OPT",   "DISP_OPT",   "OUTP_OPT",   "GROUP_OPT",  ;
	"MISC_OPT",   "ATTRIBUTES", "ACTUAL1",    "ACTUAL2",    ;
	"ACTUAL3",    "LINE_INFO",  "TEXT_INFO",  "DBF_INFO",   ;
	"GROUP_INFO", "PAGE_INFO",  "REPO_INFO",  "FIELD_INFO" }
  local cLib, cIfile, lInval

  while ( AcceptLib (4, 8, cHead, @cLib, "libName", { .t., .f. }) )
    CloseDbf ("LIB") ; V_LNAME := [] ; lInval := .f.

    if ( OpenDbf (cLib, "LIB") )
      LIB->(AEval (aChkStru, ;
                   {|cFld| lInval := lInval .or. ;
                                     Type ("FIELD->" + cFld) == 'U'}))

      if ( ! lInval )
        if ( File (cIfile := ClrPathExp (cLib, 4) + ".NTX" ) )
          LIB->(SetIndex (cIfile)) ; end

        if ( Upper (LIB->(IndexKey(0))) != "UPPER(REPORT)+STR(FIELD_NO,4)" )
          ClrMessage (cMess, 3)

          if ( LIB->(OpenDbf (cLib, "LIB", .f., .t.)) )
            LIB->(IndexOn (UPPER(REPORT)+STR(FIELD_NO,4), cIfile))
            LIB->(OpenDbf (cLib, "LIB", .f., .f., cIfile))
          else
            ClrNetErr () ; CloseDbf ("LIB")
          end

          ClrWindow ()	/* wait message */
        end
      else ; CloseDbf ("LIB") ; end
    end

    ClrWindow ()	/* accept window */

    if ( ! Empty (select ("LIB")) )
      V_LNAME := cLib ; exit
    elseif ( lInval ) ; ClrMessage (cErr1, 2)
    elseif ( NetErr () ) ; ClrNetErr ()
    else ; ClrMessage (cErr2, 2) ; end
  end

return NIL


**********< Create new library >***********************************************

static func LibNew ()
  
#ifdef GERMAN
  static cHead  := "Eingabe: Pfad und Name der neuen Bibliothek > F10 Liste", ;
	 cMess  := "Bibliothek wird angelegt"
#else
  static cHead  := "Input: Path and Filename > F10 Selection", ;
	 cMess  := "Creating library"
#endif
  static aLibStru := { { "REPORT",     "C",  15, 0 }, ;
		       { "DBF_ALIAS",  "C",  10, 0 }, ;
		       { "FIELD_NO",   "N",   4, 0 }, ;
		       { "FIELD_NAME", "C",  25, 0 }, ;
		       { "FIELD_EXPR", "C", 100, 0 }, ;
		       { "FIELD_TYPE", "C",   1, 0 }, ;
		       { "FIELD_LEN",  "N",   3, 0 }, ;
		       { "FIELD_DEC",  "N",   2, 0 }, ;
		       { "FORM_OPT",   "C",   1, 0 }, ;
		       { "DISP_OPT",   "C",   1, 0 }, ;
		       { "OUTP_OPT",   "C",   1, 0 }, ;
		       { "GROUP_OPT",  "C",   2, 0 }, ;
		       { "MISC_OPT",   "C",   1, 0 }, ;
		       { "ATTRIBUTES", "C",  14, 0 }, ;
		       { "ACTUAL1",    "L",   1, 0 }, ;
		       { "ACTUAL2",    "L",   1, 0 }, ;
		       { "ACTUAL3",    "L",   1, 0 }, ;
		       { "LINE_INFO",  "M",  10, 0 }, ;
		       { "TEXT_INFO",  "M",  10, 0 }, ;
		       { "DBF_INFO",   "M",  10, 0 }, ;
		       { "GROUP_INFO", "M",  10, 0 }, ;
		       { "PAGE_INFO",  "M",  10, 0 }, ;
		       { "REPO_INFO",  "M",  10, 0 }, ;
		       { "FIELD_INFO", "M",  10, 0 } }
  local cLib

  while ( AcceptLib (4, 8, cHead, @cLib, "libNew", { .f., .f. }) )
    CloseDbf ("LIB") ; V_LNAME := []

    ClrMessage (cMess, 3)

    DBCreate (cLib, aLibStru)

    if ( OpenDbf (cLib, "LIB", .t., .t.) )
      LIB->(IndexOn (UPPER(REPORT)+STR(FIELD_NO,4), ClrPathExp (cLib, 4)+".NTX"))
      LIB->(OpenDbf (cLib, "LIB", .f., .f., ClrPathExp (cLib, 4)+".NTX"))
    end

    ClrWindow ()	/* wait message */
    ClrWindow ()	/* accept window */

    if ( ! Empty (select ("LIB")) )
      V_LNAME := cLib ; exit
    else ; ClrNetErr () ; end
  end

return NIL
  

**********< Rename library >***************************************************

static func LibRen ()

#ifdef GERMAN
  static cHead1 := ;
	    "Eingabe: Pfad und Name der umzunennenden Bibliothek > F10 Liste",;
	 cHead2 := ;
	    "Eingabe: Neuer Name fr die Bibliothek > F10 Liste"
#else
  static cHead1 := "Input: old Path and Filename > F10 Selection", ;
	 cHead2 := "Input: new Path and Filename > F10 Selection"
#endif
  local cLib, cNew, cBase, lDone := .f.

  while ( AcceptLib (4, 8, cHead1, @cLib, "libName", { .t., .t. }) )
    if ( lDone := AcceptLib (7, 14, cHead2, @cNew, "libNew", { .f., .f. }) )
      rename (cLib) to (cNew) ; cBase := ClrPathExp (cLib, 4)
      rename (cBase + sx_MemoExt()) to (ClrPathExp (cNew, 4) + sx_MemoExt())
      rename (cBase + ".NTX") to (ClrPathExp (cNew, 4) + ".NTX")
      ClrWindow ()	/* accept window */
    end

    ClrWindow ()	/* accept window */
    if ( lDone ) ; exit ; end
  end

return NIL
  

**********< Delete library >***************************************************

static func LibDel ()

#ifdef GERMAN
  local cHead   := ;
           "Eingabe Pfad und Name der zu lschenden Bibliothek > F10 Liste", ;
	bQuery  := ;
	   {|cNam| "Alle Reports in " + ClrPathExp (cNam, 3) + " lschen"}, ;
	aPrompt := { "Abbrechen", "Lschen" }
#else
  local cHead   := "Input: Path and Filename > F10 Selection", ;
	bQuery  := ;
	   {|cNam| "Erase all reports in " + ClrPathExp (cNam, 3)}, ;
	aPrompt := { "Abort" , "Continue" }
#endif
  local cLib, lDone := .f.

  while ( AcceptLib (4, 8, cHead, @cLib, "libName", { .t., .t. }) )
    if ( ClrQuery (7, 14, .t., CLRCOLOR[_C_WARNING], Eval (bQuery, cLib), ;
	 aPrompt, 1) == 2 )
      erase (ClrPathExp (cLib, 4) + ".NTX")
      erase (ClrPathExp (cLib, 4) + sx_MemoExt())
      erase (cLib) ; lDone := .t.
    end

    ClrWindow ()	/* accept window */
    if ( lDone ) ; exit ; end
  end

return NIL
  

**********< Reorganize library >***********************************************

static func LibPack ()

#ifdef GERMAN
  local cQuery := "Bibliothek " + ClrPathExp (V_LNAME, 3) + ;
			" reorganisieren", ;
	cMess  := "Bibliothek wird reorganisiert"
#else
  local cQuery := "Reorganize library " + ClrPathExp (V_LNAME, 3), ;
	cMess  := "Reorganizing data"
#endif
  local cTmpDbf

  if ( ClrYNQuery (9, 20, .t., /* color */, cQuery, .t.) )
    if ( LIB->(OpenDbf (V_LNAME, "LIB", .f., .t.)) )
      ClrMessage (cMess, 3)

      if ( (cTmpDbf := TempDbf ("LIB", "LIBPACK", ClrPathExp (V_LNAME, 1), ;
			        "DBF", .t.)) != [] )
        CloseDbf ("LIB") ; erase (V_LNAME)		/* erase old cLib */
        erase (ClrPathExp (V_LNAME, 4) + ".NTX")
        erase (ClrPathExp (V_LNAME, 4) + sx_MemoExt())

        LIBPACK->(DBEval ({ || actual2 := .f. }))	/* clear net flag */

        LIBPACK->(IndexOn (UPPER(REPORT)+STR(FIELD_NO,4), ;
			   ClrPathExp (V_LNAME, 4)+".NTX"))	/* build new index */

        CloseDbf ("LIBPACK")

        rename (cTmpDbf) to (V_LNAME)			/* rename new cLib */
        rename (ClrPathExp (cTmpDbf, 4) + sx_MemoExt()) to ;
               (ClrPathExp (V_LNAME, 4) + sx_MemoExt())

        if ( ! OpenDbf (V_LNAME, "LIB", .t., .f., 	/* open lib again */ ;
		        ClrPathExp (V_LNAME, 4)+".NTX") )
          ClrNetErr () ; V_LNAME := [] ; end
      else
        CloseDbf ("LIB") ; V_LNAME := [] ; ClrNetErr ()
      end

      ClrWindow ()	/* wait window */
    else
      if ( ! OpenDbf (V_LNAME, "LIB", NIL, NIL, ;
		      ClrPathExp (V_LNAME, 4) + ".NTX") )
        V_LNAME := [] ; ShowState () ; end
      ClrNetErr ()
    end
  end

return NIL


**********< Get report from library >******************************************

static func ReportGet ()

#ifdef GERMAN
  static cError := "Netzwerk Fehler / Datei reorganisieren"
#else
  static cError := "Network error / database corrupted"
#endif
  local cReport

  if ( ChooseRepo (4, 26, .t., @cReport) )
    LIB->(Seek (cReport))

    if ( LIB->actual2 .or. ! LIB->(RLock ()) )	/* head record locked ? */
      ClrMessage (cError, 2)
    else
      LIB->actual2 := .t. ; V_RNAME := cReport
    end

    LIB->(Unlock ())
  end

return NIL					/* .t. ==> edit report */


**********< Create new report >************************************************

static func ReportNew ()

#ifdef GERMAN
  static cHead := "Eingabe: Name des neuen Reports > F10 Liste"
#else
  static cHead := "Input: New Report Name > F10 Selection"
#endif
  local cReport

  if ( AcceptRepo (4, 26, cHead, @cReport, "repoNew") )
    if ( LIB->(Append (NIL, {|| report := cReport, actual2 := .t.})) )
      V_RNAME := Trim (cReport) ; LIB->(Unlock ())
    else
      ClrNetErr ()
    end
  end

return NIL
  

**********< Rename report >****************************************************

static func ReportRen ()

#ifdef GERMAN
  local bHead  := ;
	   { |cName| "Eingabe: Neuer Name fr " + cName + " > F10 Liste" }, ;
	cMess  := "Report wird umbenannt"
#else
  local bHead  := ;
	   { |cName| "Input: New Name for " + cName + " > F10 Selection" }, ;
	cMess  := "Renaming report"
#endif
  local cReport, cNewRepo, svOrder

  while ( ChooseRepo (4, 26, .t., @cReport) )
    if ( AcceptRepo (4, 12, Eval (bHead, cReport), @cNewRepo, "repoNew") )
      LIB->(Seek (cReport))

      if ( LIB->actual2 .or. ! LIB->(FLock ()) )
        ClrNetErr ()
      else
        ClrMessage (cMess, 3)

        svOrder := LIB->(SetOrder (0))
        LIB->(DBEval ({|| report := cNewRepo }, ;
		      {|| Trim (report) == cReport }))
        LIB->(SetOrder (svOrder))

        ClrWindow ()	/* wait message */
      end

      LIB->(Unlock ()) ; exit
    end
  end

return NIL


**********< Delete report in library >*****************************************

static func ReportDel ()

#ifdef GERMAN
  local bQuery  := { |cRepo| "Alle Daten von " + cRepo + " lschen" }, ;
	aPrompt := { "Abbrechen", "Lschen" }
#else
  local bQuery  := { |cRepo| "Erase all data in " + cRepo }, ;
	aPrompt := { "Abort", "Continue" }
#endif
  local cReport

  if ( ChooseRepo (4, 26, .t., @cReport) )
    if ( ClrQuery (4, 12, .t., /* color */, Eval (bQuery, cReport), ;
		   aPrompt, 1) == 2 )
      LIB->(Seek (cReport))

      if ( LIB->actual2 .or. ! LIB->(FLock ()) )
        ClrNetErr ()
      else
        LIB->(Del (.f., .t., Trim (report) == cReport))
      end

      LIB->(Unlock ())
    end
  end

return NIL


**********< Copy report >******************************************************

static func ReportCopy ()

#ifdef GERMAN
  local bHead  := ;
	   { |cName| "Eingabe: Kopierziel fr " + cName + " > F10 Liste" }, ;
	cMess  := "Report wird kopiert"
#else
  local bHead  := ;
	   { |cName| "Input: Target for " + cName + " > F10 Selection" }, ;
	cMess  := "Copying report"
#endif
  local cReport, cNewRepo, cTempDbf

  while ( ChooseRepo (4, 26, .t., @cReport) )
    if ( AcceptRepo (4, 12, Eval (bHead, cReport), @cNewRepo, "repoNew") )
      if ( ! LIB->(FLock ()) )
        ClrNetErr ()
      else
        ClrMessage (cMess, 3)

        LIB->(Seek (cReport))

        if ( (cTempDbf := TempDbf ("LIB", "REPOCOPY", NIL, NIL , .t., ;
                                   Trim (report) == cReport)) != [] )
          REPOCOPY->(DBEval ({|| report := cNewRepo, actual2 := .f. }))
          CloseDbf ("REPOCOPY")

          LIB->(AppeFrom (cTempDbf)) ; LIB->(Unlock ())

	  erase (cTempDbf) ; erase (ClrPathExp (cTempDbf, 4) + sx_MemoExt())
        else
          ClrNetErr ()
        end

        ClrWindow ()	/* wait window */
      end

      exit
    end
  end

return NIL


**********< External Implemented Functions >***********************************

static func ReportOut ()		/* CLRPRINT.PRG */

  local cReport

  while ( ChooseRepo (4, 10, .f., @cReport) )
    ShowState (V_LNAME, cReport)
    ClrRepoExec (cReport)
    ShowState (V_LNAME, [])

    ClrWindow ()	/* report selection */
  end

return NIL


static func GlobPage ()			/* CLRGLOB.PRG */

return ClrGlobPage ()


static func GlobPrint ()		/* CLRGLOB.PRG */

return ClrGlobPrn ()


static func GlobGlob ()			/* CLRGLOB.PRG */

return ClrGlobGlob ()


*******************************************************************************
**********< Supplemental functions >*******************************************
*******************************************************************************

**********< Setup Support Functions >******************************************

static func Environ (aRest)		/* save / restore environment */
  
#ifdef GERMAN
  static cDateFormat := "dd.mm.yy"
#else
  static cDateFormat := "dd/mm/yy"
#endif
  static aSwitches := { ;
		{ _SET_EXACT,		.t.	}, ;
		{ _SET_FIXED,		.f.	}, ;
		{ _SET_DECIMALS,	2	}, ;
		{ _SET_DATEFORMAT,	NIL	}, ;
		{ _SET_EPOCH,		1900	}, ;
		{ _SET_PATH,		NIL	}, ;
		{ _SET_DEFAULT,		".\"	}, ;
		{ _SET_EXCLUSIVE,	.f.	}, ;
		{ _SET_SOFTSEEK,	.f.	}, ;
		{ _SET_UNIQUE,		.f.	}, ;
		{ _SET_DELETED,		.t.	}, ;
		{ _SET_CANCEL,		.f.	}, ;
		{ _SET_COLOR,		[]	}, ;
		{ _SET_CURSOR,		SC_NONE	}, ;
		{ _SET_CONSOLE,		.t.	}, ;
		{ _SET_ALTERNATE,	.f.	}, ;
		{ _SET_ALTFILE,		[]	}, ;
		{ _SET_DEVICE,		.f.	}, ;
		{ _SET_PRINTER,		.f.	}, ;
		{ _SET_PRINTFILE,	[]	}, ;
		{ _SET_MARGIN,		8	}, ;
		{ _SET_BELL,		.f.	}, ;
		{ _SET_CONFIRM,		.t.	}, ;
		{ _SET_ESCAPE,		.t.	}, ;
		{ _SET_INSERT,		.t.	}, ;
		{ _SET_EXIT,		.f.	}, ;
		{ _SET_INTENSITY,	.t.	}, ;
		{ _SET_SCOREBOARD,	.f.	}, ;
		{ _SET_DELIMITERS,	.f.	}, ;
		{ _SET_DELIMCHARS,	""	}, ;
		{ _SET_WRAP,		.t.	}, ;
		{ _SET_MESSAGE,		0	}, ;
		{ _SET_MCENTER,		.f.	}, ;
		{ _SET_SCROLLBREAK,	.f.	} }

  local aSave := {}, i := 0

  if ( aRest == NIL )
    MoreFiles ()	/* increase number of open files (DOS 3.0 - 3.2) */

    AEval (aSwitches, {|aSwitch| Aadd (aSave, Set (aSwitch[1], aSwitch[2]))})
    Aadd (aSave, SetBlink (.f.))

    Set (_SET_DATEFORMAT, cDateFormat)
    Set (_SET_PATH, ExeDir ())
  else
    AEval (aSwitches, {|aSwitch| ++i, Set (aSwitch[1], aRest[i])}, 1, ;
           len (aSwitches))
    SetBlink (aRest[len (aRest)])
  end

return aSave


static func Border ()			/* fraw frame */

#ifdef GERMAN
  static cTitle := "CLReport 3.0    Report Generator fr Clipper 5.0 / DBFSix"
  static cHelpTxt := "  F1 Hilfe"
#else
  static cTitle := "CLReport 3.0    Report Generator for Clipper 5.0 / DBFSix"
  static cHelpTxt := "  F1 Help"
#endif

  clear screen

  SelColor (2) ; @ 0, 0 ; @ 0, 2 say cTitle
  @ 0, MaxCol () - len (cHelpTxt) - 2 say cHelpTxt
  SelColor (1)

return NIL


static func ShowState (cLib, cRepo)	/* display lib and report name */

  local svColor := SetColor (CLRCOLOR[_C_FRAME])

  SelColor (3)
  @ MaxRow (), 0 say "  Lib: " + ;
		     iif (len (cLib) <= 44, ClrStrRPad (cLib, 44), ;
                          ClrStrNCut (cLib, 44)) + ;
		     "   Report: " + ;
		     ClrStrRPad (cRepo, 15) + ;
		     "   "
  SetColor (svColor)

return NIL


**********< User Interaction Support Functions >*******************************

static func AcceptLib (nRow, nCol, cHead, cInput, cName, aUserData)
					/* query library name */

  local lResult := .f.

  cInput := Space (64)

  if ( ! ClrAccept (nRow, nCol, .f., /* col */, cHead, [], @cInput, "@S45K!", ;
		    { |xValue, g, aInput, n| ;
			  iif (Empty (xValue), .f., ;
			       MainValid (xValue, g, aUserData)) }, ;
		    cName, ;
		    { |xValue, g, aInput, n| MainList (xValue, g) }) )
    ClrWindow ()	/* close accept window */
  else
    cInput := AllTrim (cInput) ; lResult := .t.
  end

return lResult


static func AcceptRepo (nRow, nCol, cHead, cInput, cName)
					/* query new report name */

  cInput := Space (Len (LIB->report))

return ClrAccept (nRow, nCol, .t., /* color */, cHead, [], @cInput, "@K!", ;
		  { |xValue, g, aInput, n| ;
			iif (Empty (xValue), .f., ;
			     MainValid (xValue, g, NIL)) }, ;
		  cName, ;
	          { |xValue, g, aInput, n| MainList (xValue, g) })


static func ChooseRepo (nRow, nCol, lClose, cReport)
					/* select existent report */

#ifdef GERMAN
  local cError := "Kein Report in Bibliothek " + ClrPathExp (V_LNAME, 3) + ;
		  " vorhanden", ;
	cMess  := "Reportauswahl wird geladen", ;
	cHead  := "Reportauswahl"
#else
  local cError := "No reports found in library " + ClrPathExp (V_LNAME, 3), ;
	cMess  := "Loading report selection", ;
	cHead  := "Report Selection"
#endif
  local aReps := {}, nChoice := 0

  if ( ! LIB->(GoTop ()) )		// EOF
    ClrMessage (cError, 1) ; cReport := []
  else
    ClrMessage (cMess, 3)

    repeat
      Aadd (aReps, Upper (Trim (LIB->report)))
    until ( ! LIB->(Seek (Upper (LIB->report) + "::::", .t.)) )

    ClrWindow ()	/* wait window */

    if ( (nChoice := ClrAchoice ({ nRow,nCol, NIL,NIL }, lClose, /* color */, ;
				 cHead, /* foot */, /* flag */, aReps)) != 0 )
      cReport := Upper (Trim (aReps[nChoice]))
    else
      if ( ! lClose ) ; ClrWindow () ; end ; cReport := []
    end
  end

return nChoice != 0


static func MainValid (cValue, g, aUserData)	/* validation of gets */

#ifdef GERMAN
  local cError1 := ClrStrNCut (Trim (cValue), 20) + " ist nicht zulssig", ;
	cError2 := ClrStrNCut (Trim (cValue), 20) + " ist nicht vorhanden",;
	cError3 := ClrStrNCut (Trim (cValue), 20) + " bereits vorhanden", ;
	cError4 := ClrStrNCut (Trim (cValue), 20) + " ist noch geffnet"
#else
  local cError1 := ClrStrNCut (Trim (cValue), 20) + " is invalid", ;
	cError2 := ClrStrNCut (Trim (cValue), 20) + " not found", ;
	cError3 := ClrStrNCut (Trim (cValue), 20) + " already exists", ;
	cError4 := ClrStrNCut (Trim (cValue), 20) + " is still open"
#endif
  local lExist, cError

  do case
    case g:name == "libNew" .or. g:name == "libName"
      if ( " " $ AllTrim (cValue) .or. "*" $ cValue .or. "?" $ cValue .or. ;
           Empty (cValue := ClrPathExp (cValue)) )
        cError := cError1
      else
        if ( ! '.' $ ClrPathExp (cValue, 2) ) ; cValue += ".CLR" ; end

        g:varPut (cValue)

        if ( (lExist := aUserData[1]) != NIL )	/* existence flag */
          if ( lExist )
            if ( ! File (cValue) .or. ;
                 ! File (ClrPathExp (cValue, 4) + sx_MemoExt()) )
              cError := cError2 ; end
          else
            if ( File (cValue) .or. ;
                 File (ClrPathExp (cValue, 4) + sx_MemoExt()) )
              cError := cError3 ; end
          end
        end

        if ( cError == NIL .and. aUserData[2] )	/* must be closed */
          if ( ! OpenDbf (cValue, "LIBCHECK", .t., .t.) )
            cError := cError4
          else ; CloseDbf ("LIBCHECK") ; end
        end
      end

    case g:name == "repoNew"
      if ( ! Empty (StrChRepl (Upper (cValue), ;
                               "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789$@_", ;
                               "                                       ")) )
        cError := cError1
      elseif ( LIB->(Seek (cValue)) )
        cError := cError3 ; end
  end

  if ( cError != NIL ) ; ClrMessage (cError, 2) ; end

return cError == NIL


static func MainList (xValue, g)	/* selections F10 */

  local cVar

  do case
    case g:name == "libName" .or. g:name == "libNew"
      if ( Empty (cVar := ClrSDir ("*.CLR", AllTrim (xValue))) .or. ;
           g:name != "libName" )
        cVar := NIL ; end

    case g:name == "repoNew"
      ChooseRepo (7, 40, .t., @cVar) ; cVar := NIL
  end

return cVar
 

**********< Help F1 >**********************************************************

static func Help (cProc, nLine, cInvar)

#ifdef GERMAN
  static cError := "Hilfedatei fehlt", ;
	 cQuery := "Keine Hilfestellung vorhanden. Hilfetext einrichten", ;
	 cHead  := "Hilfe-System", ;
	 cFoot  := ;
	    "ESC verlassen  -  ALT-F1 edieren  ALT-F5 lschen  ALT-F10 reorg"
#else
  static cError := "Help File not found", ;
	 cQuery := "No help available. Create help screen", ;
	 cHead  := "Online-Help", ;
	 cFoot  := ;
	    "ESC leave  -  ALT-F1 edit  ALT-F5 delete  ALT-F10 reorganize"
#endif
  local svCursor, svKey, svRow := Row (), svCol := Col (), lEdit := .f.

if ( ! File (HelpFile + ".DBF") .or. ! File (HelpFile + sx_MemoExt()) .or. ;
     ! File (HelpFile + IndexExt()) )
    ClrMessage (cError, 1)
  elseif ( ! OpenDbf (HelpFile, "HELP", .t., .f., HelpFile + IndexExt()) )
    ClrNetErr ()
  else
    svCursor = SetCursor (SC_NONE) ; svKey := SetKey (K_F1, NIL)
    Set Order To 1

    if ( ! HELP->(Seek (V_HSTW)) )
      if ( ClrYNQuery (NIL, NIL, .t., /* color */, cQuery) )
        if ( ! HELP->(Append (NIL, {|| FIELD->stw := V_HSTW })) )
          ClrNetErr ()
        else ; lEdit := .t. ; end
      end
    end

    if ( Trim (HELP->stw) == V_HSTW )	/* start editor */
      ClrMiniEdit ({ 2, 4, 22, 75 }, .t., CLRCOLOR[_C_HELP], cHead, cFoot, ;
		   /* flags */, HELP->inhalt, ;
                   {|editor, nPCode, nSCode| ;
			HelpCallback (editor, nPCode, nSCode, @lEdit)})
    end

    SetCursor (svCursor) ; SetKey (K_F1, svKey)
    CloseDbf ("HELP")
  end

  SetPos (svRow, svCol)

return NIL


static func HelpCallback (editor, nPCode, nSCode, lEdit)

#ifdef GERMAN
  static cQuery1 := "Hilfe-Eintrag lschen", ;
	 cQuery2 := "nderungen speichern", ;
	 cQuery3 := "Hilfedatei reorganisieren", ;
	 cHead1  := "Dateiname"
#else
  static cQuery1 := "Delete help entry", ;
	 cQuery2 := "Save changes", ;
	 cQuery3 := "Reorganize help database", ;
	 cHead1  := "Filename"
#endif
  local nResult := 0, cFile

  field STW

  do case
    case nPCode == TCallbackPostkey
      do case
        case nSCode == K_ALT_F1
          if ( HELP->(RLock ()) )
            editor:insertState := editor:mode := lEdit := .t.
          else ; ClrNetErr () ; end
        case nSCode == K_ALT_F5
          if ( ClrYNQuery (editor:nTop + 2, editor:nLeft + 4, .t., NIL, ;
			   cQuery1, .f.) )
             if ( HELP->(Del ()) )
               editor:mode := NIL ; lEdit := .f. ; editor:leave := .t.
             else ; ClrNetErr () ; end
          end
        case nSCode == K_ALT_F10
          if ( editor:mode == NIL .and. ;
               ClrYNQuery (editor:nTop + 2, editor:nLeft + 4, .t., NIL, ;
			   cQuery3, .t.) )
            if ( HELP->(OpenDbf (HelpFile, "HELP", .f., .t.)) )
              HELP->(Pack ()) ; HELP->(IndexOn (STW, HelpFile+IndexExt()))
            else ; ClrNetErr () ; end

            HELP->(OpenDbf (HelpFile, "HELP", .f., .f., HelpFile+IndexExt()))
            HELP->(Seek (V_HSTW))
          end
        case (nSCode == K_ALT_R .or. nSCode == K_ALT_W) .and. ;
	     editor:mode != NIL
          cFile := Space (50)

          if ( ClrAccept (NIL, NIL, .t., NIL, cHead1, [], @cFile, "@!") )
            editor:block (cFile, nSCode == K_ALT_W) ; end
        otherwise
          nResult := nSCode
      end

    case nPCode == TCallbackStart
      if ( lEdit ) ; editor:mode := lEdit ; end

    case nPCode == TCallbackSaveText .or. nPCode == TCallbackEnd
      if ( lEdit .and. ! Deleted () )
        if ( ClrYNQuery (editor:nTop + 2, editor:nLeft + 4, .t., NIL, ;
			 cQuery2, .t.) )
          HELP->inhalt := editor:text
        end
      end

    otherwise
      nResult := TextCallback (editor, nPCode, nSCode)
  end

return nResult


/*---------------------------< EOF clrmain.prg >-----------------------------*/
